require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.os.*"
import "android.graphics.Color"
import "android.content.Intent"
import "android.net.Uri"
import "android.content.Context"
import "android.media.MediaPlayer"
import "java.io.File"

activity.setTitle("Number Pad Keyboard")
activity.setTheme(android.R.style.Theme_Material_Light)

layout = LinearLayout()
layout.setOrientation(LinearLayout.VERTICAL)
layout.setPadding(20, 20, 20, 20)
layout.setBackgroundColor(Color.WHITE)

inputText = ""

displayLayout = LinearLayout()
displayLayout.setOrientation(LinearLayout.VERTICAL)
displayLayout.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT))
displayLayout.setPadding(10, 10, 10, 30)

textDisplay = TextView()
textDisplay.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT))
textDisplay.setText("")
textDisplay.setTextSize(24)
textDisplay.setTextColor(Color.BLACK)
textDisplay.setGravity(Gravity.CENTER)
displayLayout.addView(textDisplay)

layout.addView(displayLayout)

numpadLayout = LinearLayout()
numpadLayout.setOrientation(LinearLayout.VERTICAL)
numpadLayout.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT))
numpadLayout.setGravity(Gravity.CENTER)

function getAppDirectory()
  return activity.getLuaDir()
end

function createAudioFolder()
  local folderPath = getAppDirectory() .. "/audio"
  local folder = File(folderPath)
  
  if not folder.exists() then
    folder.mkdirs()
  end
  
  return folderPath
end

function playButtonAudio(buttonName)
  local mediaPlayer = MediaPlayer()
  local folderPath = createAudioFolder()
  local audioFilePath = folderPath .. "/" .. buttonName .. ".mp3"
  local audioFile = File(audioFilePath)

  if audioFile.exists() then
    xpcall(function()
      mediaPlayer.setDataSource(audioFilePath)
      mediaPlayer.prepare()
      mediaPlayer.start()
      mediaPlayer.setOnCompletionListener{
        onCompletion = function(mp)
          mp.release()
        end
      }
    end, function(err)
      Toast.makeText(activity, "آڈیو پلے نہیں ہو سکی: " .. tostring(err), Toast.LENGTH_SHORT).show()
      mediaPlayer.release()
    end)
  else
    Toast.makeText(activity, "آڈیو فائل نہیں ملی: " .. buttonName .. ".mp3", Toast.LENGTH_SHORT).show()
  end
end

function showDeleteNotification(deletedChar)
  local displayText = deletedChar

  if deletedChar == "*" then
    displayText = "ASTERISK"
  elseif deletedChar == "#" then
    displayText = "HASH"
  end

  Toast.makeText(activity, displayText .. " Deleted Successfully", Toast.LENGTH_SHORT).show()
end

function createRow(numbers)
  local row = LinearLayout()
  row.setOrientation(LinearLayout.HORIZONTAL)
  row.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT))
  row.setGravity(Gravity.CENTER)
  
  for i, num in ipairs(numbers) do
    local button = Button(activity)
    
    if num == "*" then
      button.setText("ASTERISK")
    elseif num == "#" then
      button.setText("HASH")
    else
      button.setText(num)
    end
    
    button.setTextSize(22)
    local params = LinearLayout.LayoutParams(150, 150)
    params.setMargins(10, 10, 10, 10)
    button.setLayoutParams(params)
    
    if num == "Delete" then
      button.setOnClickListener{
        onClick = function()
          if #inputText > 0 then
            local deletedChar = string.sub(inputText, -1)
            inputText = string.sub(inputText, 1, -2)
            textDisplay.setText(inputText)
            showDeleteNotification(deletedChar)
          end
        end
      }
    elseif num == "Call" then
      button.setOnClickListener{
        onClick = function()
          makeDirectCall()
          playButtonAudio("CALL")
        end
      }
    elseif num == "*" then
      button.setOnClickListener{
        onClick = function()
          inputText = inputText .. num
          textDisplay.setText(inputText)
          playButtonAudio("ASTERISK")
        end
      }
    else
      button.setOnClickListener{
        onClick = function()
          inputText = inputText .. num
          textDisplay.setText(inputText)
          playButtonAudio(num)
        end
      }
    end
    
    row.addView(button)
  end
  
  return row
end

numpadLayout.addView(createRow({"1", "2", "3"}))
numpadLayout.addView(createRow({"4", "5", "6"}))
numpadLayout.addView(createRow({"7", "8", "9"}))
numpadLayout.addView(createRow({"*", "0", "#"}))
numpadLayout.addView(createRow({"Call", "Delete"}))

layout.addView(numpadLayout)

function checkPermission()
  if Build.VERSION.SDK_INT >= 23 then
    local permissions = {
      "android.permission.READ_EXTERNAL_STORAGE",
      "android.permission.WRITE_EXTERNAL_STORAGE",
      "android.permission.CALL_PHONE"
    }
    activity.requestPermissions(permissions, 1)
  end
end

checkPermission()

createAudioFolder()

activity.setContentView(layout)

function makeDirectCall()
  if #inputText > 0 then
    local intent = Intent(Intent.ACTION_CALL)
    intent.setData(Uri.parse("tel:" .. inputText))
    
    xpcall(function()
      activity.startActivity(intent)
      playButtonAudio("CALL")
    end, function(err)
      Toast.makeText(activity, "کال نہیں ہو سکی: " .. tostring(err), Toast.LENGTH_LONG).show()
    end)
  else
    Toast.makeText(activity, "براہ کرم پہلے نمبر درج کریں", Toast.LENGTH_SHORT).show()
  end
end